using System;
using Jypeli;
using Jypeli.Controls;
using Jypeli.Widgets;
using System.Collections.Generic;

namespace DemoniEscape
{
    /// 
    /// @author Sakari Limma
    /// @version 2.4.2025
    /// 
    ///<summary>
    /// Arvaa luku -peli, jossa pelaaja yrittää arvata satunnaisen luvun tietyllä määrällä yrityksiä. Pelaaja
    /// on jäänyt DEMOnin vangiksi. Arvaamalla oikein hän vapautuu ja läpäisee kurssin, vastaamalla väärin
    /// hän jää ikuisesti Ohjelmointi 1 kurssille.
    /// </summary>
    public class DemoniEscape : Game
    {
        /// <summary>
        /// Pelin asetukset
        /// <summary>
        private const int MinLuku = 1;
        private const int MaxLuku = 50;
        private const int MaxYritykset = 10;

        /// <summary>
        /// Pelin muuttujat
        /// </summary>
        private int _arvattava;
        private int _yritykset;
        private Label _viesti;
        private Label _viesti2;
        private Label _viesti3;
        private InputBox _syotekentta;
        private List<int> _arvatutLuvut;

        /// <summary>
        /// Kuvareferenssit
        /// </summary>
        private static readonly Image DemoniKuva = LoadImage("DEMOni2");
        private static readonly Image PuhekuplaKuva = LoadImage("puhekupla");

        public override void Begin()
        {
            _yritykset = MaxYritykset;
            _arvattava = RandomGen.NextInt(MinLuku, MaxLuku + 1);
            _arvatutLuvut = new List<int>();

            // Luodaan pelin käyttöliittymä
            LuoKayttoliittyma();

            // Määritetään kameran tarkennus
            Camera.ZoomToLevel();
        }

        
        /// <summary>
        /// Luo pelin käyttöliittymäelementit ja lisää ne peliin.
        /// </summary>
        private void LuoKayttoliittyma()
        {
            // Viestit pelaajalle
            _viesti = new Label($"Arvaa luku väliltä {MinLuku}-{MaxLuku}")
            { TextColor = Color.Black, Position = new Vector(100, 180), Font = Font.DefaultBold };
            _viesti2 = new Label("Arvaa oikea luku tai jäät ikuisesti Ohjelmointi 1 kurssin vangiksi!")
            { TextColor = Color.Black, Position = new Vector(100, 230), Font = Font.DefaultBold };
            _viesti3 = new Label($"Sinulla on {MaxYritykset} yritystä!")
            { TextColor = Color.Black, Position = new Vector(100, 205), Font = Font.DefaultBold };

            // Lisää viestit peliin
            Add(_viesti);
            Add(_viesti2);
            Add(_viesti3);

            // Demoni- ja puhekupla
            GameObject demoni = new GameObject(350, 400) { Position = new Vector(-350, -70), Image = DemoniKuva };
            Add(demoni);

            GameObject puhekupla = new GameObject(750, 180) { Position = new Vector(100, 200), Image = PuhekuplaKuva };
            Add(puhekupla);

            // Syötekenttä pelaajan arvausta varten
            _syotekentta = new InputBox(10) { Position = new Vector(0, 0), TextColor = Color.Black, Color = Color.White };
            Add(_syotekentta);

            // Arvausnappi
            PushButton arvausNappi = new PushButton(150, 100, "Arvaa") { Position = new Vector(0, -100) };
            arvausNappi.Clicked += Tarkista;
            Add(arvausNappi);

            // Pelin taustaväri
            Level.Background.Color = Color.Red;
        }

        
        /// <summary>
        /// Tarkistaa pelaajan arvauksen ja päivittää pelitilan.
        /// </summary>
        private void Tarkista()
        {
            // Yritetään parsia pelaajan syöte
            if (!int.TryParse(_syotekentta.Text, out int arvaus))
            {
                _viesti.Text = "Syötä kelvollinen numero!";
                _viesti.TextColor = Color.Red;
                return;
            }
            
            
            /// Tarkistetaan, onko luku jo arvattu
                        if (OnkoJoArvattu(arvaus))
                        {
                            _viesti.Text = $"Olet jo arvannut luvun {arvaus}!";
                            _viesti.TextColor = Color.DarkOrange;
                            return;
                        }
            
            
            // Tallennetaan arvaus listaan
            _arvatutLuvut.Add(arvaus);

            // Tarkistetaan arvaus
            if (arvaus == _arvattava)
            {
                _viesti.Text = "Oikein! Päästän sinut läpi Ohjelmointi1 kurssista!";
                _viesti.TextColor = Color.Black;
                _viesti.Font = Font.DefaultBold;
                return;
            }

            
            // Vähennetään yrityksiä ja tarkistetaan pelin päättyminen
            _yritykset--;
            if (_yritykset == 0)
            {
                _viesti.Text = $"Hävisit! Oikea luku olisi ollut {_arvattava}. Jäät ikuisesti vangikseni!";
                _viesti.TextColor = Color.Red;
                _viesti.Font = Font.DefaultBold;
                return;
            }

            
            // Päivitetään viesti pelaajalle
            _viesti.Text = (arvaus < _arvattava) ? "Liian pieni!" : "Liian suuri!";
            _viesti.TextColor = Color.Black;
            _viesti.Font = Font.DefaultBold;

            // Näytetään arvatut luvut
            _viesti3.Text = $"Yrityksiä jäljellä: {_yritykset}\nArvatut luvut: {string.Join(", ", _arvatutLuvut)}";
            _viesti3.TextColor = Color.Black;
            _viesti3.Position = new Vector(50, -300);
            _viesti3.Font = Font.DefaultBold;

        }
        
        
        /// <summary>
        /// Tarkistaa, onko luku jo arvattu aiemmin.
        /// </summa
        private bool OnkoJoArvattu(int arvaus)
        {
            foreach (int aiempiArvaus in _arvatutLuvut) 
            { 
                if (aiempiArvaus == arvaus)
                    return true;
            }
            return false;
        }        
    }
}